import { world, TicksPerSecond,ItemStack, system, EquipmentSlot } from "@minecraft/server";
import { itemIdentifierList, entityList, zombieTransformList, distance3D, hillPostion, } from "./utility";
import { scanForOres } from "./xray.js";
import * as explo from "./custom_explosion.js";
import * as stun from "./stun.js";
import { specialAttackHandler } from "./item_components.js";
import * as weaponUtil from "./weapons/util.js"

function mobBottleHit(projectile, hitEntity) {
  if (projectile.typeId === 'sbs_mib:mob_bottle') {
    if (entityList.includes(hitEntity.typeId) && !hitEntity.hasTag("unuse")) {
      projectile.remove();
      let formattedEntity = hitEntity.typeId;
      formattedEntity = formattedEntity.replace('minecraft:', '');
      formattedEntity += "_bottle";
      hitEntity.runCommand(`loot spawn ~ ~ ~ loot "sbs/mib/${formattedEntity}"`);
      hitEntity.remove();
    }
    else if (hitEntity.typeId === "minecraft:iron_golem") {
      hitEntity.runCommand(`execute if entity @p run damage @s 0 projectile entity @p`);
      hitEntity.runCommand(`say Really !!! you thought you could contain a golem?`);
    }
    if (projectile.isValid()) projectile.triggerEvent("sbs_mib:despawn")
  }
}

function handleBatBottleProjectile(hitEntity) {
  hitEntity.runCommand(`summon minecraft:bat ~~1~`);
  hitEntity.runCommand(`/scriptevent sbs_mib:bat_bottle_projectile`);
}

function handlePiglinBruteBottleProjectile(hitEntity) {
  hitEntity.runCommand(`summon sbs_mib:piglin_brute_bottle_projectile_util ~~1~`);
}

world.afterEvents.projectileHitEntity.subscribe(data => {
  const projectile = data.projectile;
  const hitEntity = data.getEntityHit()?.entity;

  if (projectile.isValid()) {
    switch (projectile.typeId) {
      case "sbs_mib:bat_bottle_projectile":
        if (hitEntity) {
          handleBatBottleProjectile(hitEntity);
        }
        break;
      case "sbs_mib:piglin_brute_bottle_projectile":
        if (hitEntity) {
          handlePiglinBruteBottleProjectile(hitEntity);
        }
        break;
      default:
        mobBottleHit(projectile, hitEntity);
        break;
    }
  }
});

world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
  itemComponentRegistry.registerCustomComponent("sbs_mib:special_attack_handler", specialAttackHandler)
});

world.afterEvents.entitySpawn.subscribe((data) => {

  const itemEntities = world.getDimension("overworld").getEntities({ type: "minecraft:item" });
  const itemEntitiesN = world.getDimension("nether").getEntities({ type: "minecraft:item" });
  const itemEntitiesE = world.getDimension("the_end").getEntities({ type: "minecraft:item" });

  itemEntities.push(...itemEntitiesN, ...itemEntitiesE);

  for (const i of itemEntities) {
    const item = i.getComponent("minecraft:item").itemStack;
    const [namespace, identifier] = item.typeId.split(":");

    if (namespace === "sbs_mib") {
      if (Object.keys(itemIdentifierList).includes(identifier)) {
        const summonedMob = itemIdentifierList[identifier];
        if (summonedMob) {
          for (let t = 1; t <= 5; t++) {
            system.runTimeout(() => {
              if (i.isValid()) {
                i.runCommand(`particle sbs_mib:item_particle${t} ~ ~ ~`);
              }
            }, TicksPerSecond * t);
          }
          system.runTimeout(() => {
            if (i.isValid()) {
              i.runCommand(`summon ${summonedMob} ~ ~ ~`);
              i.runCommand(`particle minecraft:large_explosion ~ ~1 ~`);
              i.remove();
            }
          }, TicksPerSecond * 6);
        }
      }
    }
  }
});

const eventHandlers = {
  "sbs_mib:create_explosion": (event) => explo.createCustomExplosion(event),
  "sbs_mib:explosion_knockback": (event) => explo.explosionKnockback(event),
  "sbs_mib:stun": (event) => stun.stunEventHandler(event),
  "sbs_mib:pull_entities": (event) => weaponUtil.pullEntities(event),
  "sbs_mib:zombie_transform": (event) => {
    try {
      const { sourceEntity, id, sourceType } = event;
      if (sourceEntity && sourceEntity.typeId) {
        // Logika transformacije zombija
        const sourceEntityName = sourceEntity.typeId.split(":")[1];
        if (zombieTransformList.hasOwnProperty(sourceEntityName)) {
          sourceEntity.runCommand(`summon ${zombieTransformList[sourceEntityName]}`);
          sourceEntity.remove();
        }
      }
    }
    catch { }
  },

  "sbs_mib:set_ice": (event) => handleSetBlock(event, "minecraft:ice"),
  "sbs_mib:set_snow": (event) => handleSetBlock(event, "minecraft:powder_snow"),

  "sbs_mib:lava_floor": (event) => {
    let i = 4;
    system.runInterval(() => {
      if (i > 0) {
        world.sendMessage(`§cThe floor will be lava in ${i} seconds`);
        i--;
      }
    }, TicksPerSecond * 1);
  },

  "sbs_mib:bottler_gun_anim": (event) => {
    const players = world.getAllPlayers();
    // Implementacija za bottler gun animaciju
  }
};

system.afterEvents.scriptEventReceive.subscribe(event => {
  for (const [prefix, handler] of Object.entries(eventHandlers)) {
    if (event.id.startsWith(prefix)) {
      try {
        handler(event); // Poziva odgovarajući handler
      } catch (error) {
        console.log(`Greška u obradi eventa ${event.id}`);
        console.log(error);
      }
      break;
    }
  }
});

function handleSetBlock(event, blockType) {
  const { sourceEntity, id } = event;
  const center = sourceEntity.location;
  hillPostion.forEach(position => {
    const { radius, yOffset, delayMultiplier } = position;
    system.runTimeout(() => {
      let levelRadius = radius;
      for (let k = 0; k < radius; k++) {
        for (let i = -levelRadius; i <= levelRadius; i++) {
          for (let j = -levelRadius; j <= levelRadius; j++) {
            let block = sourceEntity.dimension.getBlock({ x: center.x + i, y: center.y + k, z: center.z + j });
            let levelCenter = { x: center.x, y: center.y + k + yOffset, z: center.z };
            let dist = distance3D(levelCenter, block);
            if (block.isValid() && dist < levelRadius - 0.5) {
              sourceEntity.runCommand(`/execute unless block ${block.x} ${block.y} ${block.z} bedrock run setblock ${block.x} ${block.y} ${block.z} ${blockType}`);
            }
          }
        }
        levelRadius--;
      }
    }, TicksPerSecond * delayMultiplier);
  });
}

system.runInterval(() => {
  const players = world.getAllPlayers();


  const guardianBottles = world.getDimension("overworld").getEntities({ type: "sbs_mib:guardian_bottle" });
  const guardianBottlesN = world.getDimension("nether").getEntities({ type: "sbs_mib:guardian_bottle" });
  const guardianBottlesE = world.getDimension("the_end").getEntities({ type: "sbs_mib:guardian_bottle" });
  guardianBottles.push(...guardianBottlesN, ...guardianBottlesE);

  const haystacks = world.getDimension("overworld").getEntities({ type: "sbs_mib:haystack" });
  const haystacksN = world.getDimension("nether").getEntities({ type: "sbs_mib:haystack" });
  const haystacksE = world.getDimension("the_end").getEntities({ type: "sbs_mib:haystack" });
  haystacks.push(...haystacksN, ...haystacksE);

  const snowballs = world.getDimension("overworld").getEntities({ type: "sbs_mib:snowball" });
  const snowballsN = world.getDimension("nether").getEntities({ type: "sbs_mib:snowball" });
  const snowballsE = world.getDimension("the_end").getEntities({ type: "sbs_mib:snowball" });
  snowballs.push(...snowballsN, ...snowballsE);

  const armadillo_snowballs = world.getDimension("overworld").getEntities({ type: "sbs_mib:armadillo_snowball" });
  const armadillo_snowballsN = world.getDimension("nether").getEntities({ type: "sbs_mib:armadillo_snowball" });
  const armadillo_snowballsE = world.getDimension("the_end").getEntities({ type: "sbs_mib:armadillo_snowball" });
  armadillo_snowballs.push(...armadillo_snowballsN, ...armadillo_snowballsE);

  const turtles = world.getDimension("overworld").getEntities({ type: "sbs_mib:turtle" });
  const turtlesN = world.getDimension("nether").getEntities({ type: "sbs_mib:turtle" });
  const turtlesE = world.getDimension("the_end").getEntities({ type: "sbs_mib:turtle" });
  turtles.push(...turtlesN, ...turtlesE);

  const snifferUtility = world.getDimension("overworld").getEntities({ type: "sbs_mib:sniffer_utility" });
  const snifferUtilityN = world.getDimension("nether").getEntities({ type: "sbs_mib:sniffer_utility" });
  const snifferUtilityE = world.getDimension("the_end").getEntities({ type: "sbs_mib:sniffer_utility" });
  snifferUtility.push(...snifferUtilityN, ...snifferUtilityE);

  for (const player of players) {
    stun.playerStunHandler(player); // handle stunned players
    let mainhandItem = player.getComponent('equippable').getEquipment(EquipmentSlot.Mainhand);
    const force = player.getViewDirection();
    player.runCommand(`tp @e[type=sbs_mib:guardian_utility,c=1,r=2.5] ~ ~1 ~`);
    player.runCommand(`tp @e[type=sbs_mib:blaze_utility,c=1,r=2.5] ~ ~1 ~`);

    if (mainhandItem) {
      if (mainhandItem.typeId === "sbs_mib:bottler_gun") {
        player.playAnimation("animation.sbs_mib.bottler_gun.tp_default", { stopExpression: "!query.is_item_name_any('slot.weapon.mainhand', 0, 'sbs_mib:bottler_gun')" });
      }
    }

    if (player.hasTag("phantomBottle")) {
      player.applyKnockback(force.x, force.z, 3.0, force.y);
    }

    for (const entity of snifferUtility) {
      entity.runCommand(`execute if block ~ ~ ~ air run event entity @s sbs_mib:despawn`)
    }
  }

  for (const entity of haystacks) {
    const force = entity.getViewDirection();
    entity.runCommand(`execute if block ^ ^ ^1 air run tag @s add moveFront`);
    if (entity.hasTag("moveFront")) {
      entity.runCommand(`execute unless block ^ ^ ^1 air if block ^ ^1 ^1 air run tag @s add frontUp`);
      entity.runCommand(`execute unless block ^ ^ ^1 air unless block ^ ^1 ^1 air run tp @s ~ ~ ~ ~180 ~`);
      entity.runCommand(`execute unless block ^ ^ ^1 air if block ^ ^1 ^1 air run tag @s remove moveFront`);
      entity.applyKnockback(force.x, force.z, 0.95, -2);
    } else if (entity.hasTag("frontUp")) {
      entity.runCommand(`execute if block ^ ^ ^1 air run tag @s add moveFront`);
      entity.runCommand(`execute unless block ^ ^ ^1 air unless block ^ ^1 ^1 air run tp @s ~ ~ ~ ~180 ~`);
      entity.runCommand(`execute if block ^ ^ ^1 air run tag @s remove frontUp`);
      entity.applyKnockback(force.x, force.z, 0.95, 1);
    }

  }

  for (const entity of snowballs) {
    const force = entity.getViewDirection();
    entity.runCommand(`execute if block ^ ^ ^1 air run tag @s add moveFront`);
    if (entity.hasTag("moveFront")) {
      entity.runCommand(`execute unless block ^ ^ ^1 air if block ^ ^1 ^1 air run tag @s add frontUp`);
      entity.runCommand(`execute unless block ^ ^ ^1 air if block ^ ^1 ^1 snow_layer run tag @s add frontUp`);
      entity.runCommand(`execute unless block ^ ^ ^1 air unless block ^ ^1 ^1 air run tp @s ~ ~ ~ ~180 ~`);
      entity.runCommand(`execute unless block ^ ^ ^1 air if block ^ ^1 ^1 air run tag @s remove moveFront`);
      entity.applyKnockback(force.x, force.z, 0.95, -2);
    } else if (entity.hasTag("frontUp")) {
      entity.runCommand(`execute if block ^ ^ ^1 air run tag @s add moveFront`);
      entity.runCommand(`execute unless block ^ ^ ^1 air unless block ^ ^1 ^1 air run tp @s ~ ~ ~ ~180 ~`);
      entity.runCommand(`execute if block ^ ^ ^1 air run tag @s remove frontUp`);
      entity.applyKnockback(force.x, force.z, 0.95, 1);
    }
  }

  for (const entity of armadillo_snowballs) {
    const force = entity.getViewDirection();
    entity.runCommand(`execute if block ^ ^ ^1 air run tag @s add moveFront`);
    if (entity.hasTag("moveFront")) {
      entity.runCommand(`execute unless block ^ ^ ^1 air if block ^ ^1 ^1 air run tag @s add frontUp`);
      entity.runCommand(`execute unless block ^ ^ ^1 air unless block ^ ^1 ^1 air run tp @s ~ ~ ~ ~180 ~`);
      entity.runCommand(`execute unless block ^ ^ ^1 air if block ^ ^1 ^1 air run tag @s remove moveFront`);
      entity.applyKnockback(force.x, force.z, 0.95, -2);
    } else if (entity.hasTag("frontUp")) {
      entity.runCommand(`execute if block ^ ^ ^1 air run tag @s add moveFront`);
      entity.runCommand(`execute unless block ^ ^ ^1 air unless block ^ ^1 ^1 air run tp @s ~ ~ ~ ~180 ~`);
      entity.runCommand(`execute if block ^ ^ ^1 air run tag @s remove frontUp`);
      entity.applyKnockback(force.x, force.z, 0.95, 1);
    }
  }

  for (const entity of turtles) {
    const force = entity.getViewDirection();
    entity.applyKnockback(force.x, force.z, 3, -2);
    entity.runCommand(`execute at @e[type=sbs_mib:turtle] unless block ^ ^ ^1 air run loot spawn ~ ~ ~ loot "sbs/mib/turtle"`);
    entity.runCommand(`execute at @e[type=sbs_mib:turtle] unless block ^ ^ ^1 air run particle sbs_mib:despawn_particle ~ ~ ~`);
    entity.runCommand(`execute at @e[type=sbs_mib:turtle] unless block ^ ^ ^1 air run event entity @s sbs_mib:despawn`);
  }
});

world.afterEvents.itemStartUse.subscribe((data) => {
  try {
    const item = data.itemStack.typeId;
    const player = data.source;

    const itemActions = {
      "sbs_mib:shulker_bottle_item": () => handleSummon("sbs_mib:shulker_door", "^^^1 facing ^^^4", "sbs_mib:put"),
      "sbs_mib:ore_detector": () => handleScanOre(),
      "sbs_mib:turtle_bottle_item": () => handleSummon("sbs_mib:turtle", "^^0.5^1 facing ^^^4", "sbs_mib:empty"),
      "sbs_mib:phantom_bottle_item": () => handleTimedTag("phantomBottle", 5),
      "sbs_mib:horse_bottle_item": () => handleAnimalBottle("horseBottle", "sbs_mib:horse", "sbs_mib:horse_tamed", "horseTeleport", 5),
      "sbs_mib:skeleton_horse_bottle_item": () => handleAnimalBottle("skeletonHorseBottle", "sbs_mib:skeleton_horse", "sbs_mib:horse_tamed", "skeletonHorseTeleport", 10),
      "sbs_mib:zombie_horse_bottle_item": () => handleAnimalBottle("zombieHorseBottle", "sbs_mib:zombie_horse", "sbs_mib:horse_tamed", "zombieHorseTeleport", 10),
      "sbs_mib:cat_bottle_item": () => handleTimedTag("catBottle", 6),
      "sbs_mib:fox_bottle_item": () => handleTimedTag("foxBottle", 1.75),
      "sbs_mib:camel_bottle_item": () => handleSummon("sbs_mib:camel_utility", "^^^2 facing ^^^4", "sbs_mib:put"),
      "sbs_mib:piglin_bottle_item": () => handleSummon("sbs_mib:golden_block", "^^^2 facing ^^^4", "sbs_mib:put"),
      "sbs_mib:bee_bottle_item": () => handleSummon("sbs_mib:beehave_block", "^^^2 facing ^^^4", "sbs_mib:put"),
      "sbs_mib:elder_guardian_bottle_item": () => handleFillWaterArea(-17, 17),
      "sbs_mib:hoglin_bottle_item": () => handleSummonWithRotation("sbs_mib:hoglin_chair", "^^0.5^2 facing ^^^4"),
      "sbs_mib:book_item": () => handleSummonBook("sbs_mib:book"),
      "sbs_mib:allay_bottle_item": () => handleTimedEffects("allayBottle", "@a[tag=allayBottle]", "levitation", 7, 5, "resistance", 3, 255),
      "sbs_mib:ravager_bottle_item": () => handleGiveItem("sbs_mib:ravager_ram", 1),
      "sbs_mib:sniffer_bottle_item": () => handleGiveItem("sbs_mib:ore_detector", 15),
      "sbs_mib:zombified_piglin_bottle_item": () => handleSummon("sbs_mib:zombified_piglin", "^^0.5^2 facing ^^^4", "sbs_mib:put"),
      "sbs_mib:panda_bottle_item": () => handlePandaBottle("pandaBottle", "sbs_mib:plush"),
      "sbs_mib:silverfish_bottle_item": () => handleFillAreaWithReplacement("infested_stone", "stone"),
      "sbs_mib:vex_bottle_item": () => handleSummon("sbs_mib:vex", "^ ^ ^1", "sbs_mib:put"),
      "sbs_mib:parrot_bottle_item": () => handlePlayRandomSounds("sbs_mib:random_parrot_bottle", 4),
      "sbs_mib:wither_bottle_item": () => handleWitherStructureLoad("sbs_mib:custom_explosion"),
      "sbs_mib:dolphin_bottle_item": () => handlePlaySound("sbs_mib:dolphin_bottle"),
      "sbs_mib:dolphin_bottle_item": () => handleEffectAndDolphin(""),
      "sbs_mib:frog_bottle_item": () => handleEffectAndClear("levitation", 1, 38, 0.25),
      "sbs_mib:goat_bottle_item": () => handleTaggedSummonAndInterval("knockback", "knockback_finish", "sbs_mib:knockback_utility", "speed", 5.0, 5.0, 0.1, 5.0, 5.2),
      "sbs_mib:glow_squid_bottle_item": () => handleLightBlockAroundPlayer(30, 5),
      "sbs_mib:vindicator_bottle_item": () => handleSummonAxesInArea(["acacia", "birch", "dark_oak", "jungle", "oak", "spruce", "cherry", "mangrove"], 7, 7, 7),
      "sbs_mib:wither_skeleton_bottle_item": () => handleTeleportWithIntervalAndSetBlock("nether_brick", 50, 0.1),
      "sbs_mib:bat_bottle_item": () => handleBatAndClear("minecraft:bat", 10, 30, "unuse"),
    };

    if (itemActions[item]) {
      itemActions[item]();
    }

    function handleScanOre() {
      player.runCommand(`loot spawn ~ ~ ~ loot "sbs/mib/ore_drop"`);
      clearItem();
    }

    function handleSummon(entity, position, sound) {
      const loc = player.location;
      player.runCommand(`execute rotated as @s rotated ~ 0 run summon ${entity} ${position}`);
      player.runCommand(`/playsound ${sound} @s ~ ~ ~ 1 1 1`);
      clearItem();
    }

    function handleTimedTag(tag, timeoutSeconds) {
      player.addTag(tag);
      clearItem();
      system.runTimeout(() => {
        player.removeTag(tag);
      }, TicksPerSecond * timeoutSeconds);
    }

    function handleAnimalBottle(tag, entityName, tameEntity, teleportTag, timeoutSeconds) {
      player.addTag(tag);
      player.runCommand(`/ride @s summon_ride ${entityName} no_ride_change ${tameEntity}`);
      player.runCommand(`/tag @e[type=${entityName},c=1] add ${teleportTag}`);
      clearItem();
      if (player.hasTag(tag)) {
        system.runTimeout(() => {
          player.removeTag(tag);
          player.runCommand(`/ride @s stop_riding`);
          player.runCommand(`/execute at @e[type=${entityName},c=1,tag=${teleportTag}] run particle sbs_mib:despawn_particle ~ ~1 ~ `);
          player.runCommand(`/execute as @s[tag=!${tag}] run tp @e[type=${entityName},c=1,tag=${teleportTag}] ~ ~-64 ~`);
        }, TicksPerSecond * timeoutSeconds);
      }
    }

    function handleFillWaterArea(min, max) {
      for (let i = -1; i <= 1; i++) {
        for (let j = -1; j <= 1; j++) {
          player.runCommand(`fill ~${i * min + min / 2 + 1} ~${min} ~${j * min + min / 2 + 1} ~${i * min + max + min / 2 - 1} ~${max} ~${j * min + max + min / 2 - 1} air replace water`);
          player.runCommand(`fill ~${i * min + min / 2 + 1} ~${min} ~${j * min + min / 2 + 1} ~${i * min + max + min / 2 - 1} ~${max} ~${j * min + max + min / 2 - 1} air replace flowing_water`);
          player.runCommand(`fill ~${i * min + min / 2} ~${min} ~${j * min + min / 2} ~${i * min + max + min / 2} ~${max} ~${j * min + max + min / 2} air replace water`);
          player.runCommand(`fill ~${i * min + min / 2} ~${min} ~${j * min + min / 2} ~${i * min + max + min / 2} ~${max} ~${j * min + max + min / 2} air replace flowing_water`);
        }
      }
      clearItem();
    }

    function handleSummonWithRotation(entity, position) {
      const rotation = player.getRotation();
      const playerY = Math.round(rotation.y / 90) * 90 + 180;
      player.runCommand(`summon ${entity} ${position} ${playerY} 0`);
      player.runCommand(`/playsound sbs_mib:put @s ~ ~ ~ 1 1 1`);
      clearItem();
    }

    function handleSummonBook(entity) {
      const rotation = player.getRotation();
      const playerY = Math.round(rotation.y / 90) * 90;
      switch (playerY) {
        case -180:
          player.runCommand(`summon ${entity} ~1 ~ ~-2 ~-180 ~`);
          break;
        case 180:
          player.runCommand(`summon ${entity} ~1 ~ ~-2 ~-180 ~`);
          break;
        case 90:
          player.runCommand(`summon ${entity} ~-2 ~ ~-1 ~-180 ~`);
          break;
        case -90:
          player.runCommand(`summon ${entity} ~2 ~ ~1 ~-180 ~`);
          break;
        default:
          player.runCommand(`summon ${entity} ~-1 ~ ~2 ~-180 ~`);
          break;
      }
      player.runCommand(`/playsound sbs_mib:put @s ~ ~ ~ 1 1 1`);
      clearItem();
    }
    function handleTimedEffects(tag, target, effect1, amplifier1, durationSeconds1, effect2, amplifier2, durationSeconds2) {
      player.addTag(tag);
      player.runCommand(`effect ${target} ${effect1} ${amplifier1} ${durationSeconds1} true`);
      player.runCommand(`/playsound sbs_mib:smoke @s ~ ~ ~ 1 1 1`);
      clearItem();
      system.runTimeout(() => {
        if (player.hasTag(tag)) {
          player.runCommand(`execute as ${target} run effect @s ${effect1} 0 0 true`);
          player.runCommand(`execute as ${target} run effect @s ${effect2} ${amplifier2} ${durationSeconds2} true`);
          player.removeTag(tag);
        }
      }, TicksPerSecond * durationSeconds1);
    }

    function handleGiveItem(item, amount) {
      player.runCommand(`give @s ${item} ${amount}`);
      player.runCommand(`/playsound sbs_mib:put @s ~ ~ ~ 1 1 1`);
      clearItem();
    }

    function handleFillAreaWithReplacement(replaceBlock, targetBlock) {
      player.runCommand(`fill ~-5 ~-5 ~-5 ~5 ~5 ~5 ${replaceBlock} replace ${targetBlock}`);
      clearItem();
    }

    function handlePlayRandomSounds(soundBaseName, soundCount) {
      for (let i = 0; i < soundCount; i++) {
        const randomNumber = Math.floor(Math.random() * 7) + 1;
        const soundName = `${soundBaseName}${randomNumber}`;
        player.runCommand(`/playsound ${soundName} @a ~ ~ ~ 1 1 1`);
      }
      clearItem();
    }

    function handleEffectAndClear(effect, level, duration, interval) {
      player.runCommand(`/effect @s ${effect} ${level} ${duration} true`);
      clearItem();
      system.runTimeout(() => {
        player.runCommand(`/effect @s ${effect} 0 0 true`);
      }, TicksPerSecond * interval);
    }

    function handleEffectAndDolphin() {
      player.runCommand(`/effect @s conduit_power 7 3 true`);
      player.runCommand(`/effect @s speed 7 3 true`);
      clearItem();
    }

    function handlePandaBottle(tag, plush) {
      player.addTag(tag);
      player.runCommand(`summon ${plush} ^ ^ ^1`);
      player.runCommand(`/playsound sbs_mib:put @s ~ ~ ~ 1 1 1`);
      clearItem();
    }

    function handleLightBlockAroundPlayer(radius, interval) {
      for (let x = -radius; x <= radius; x += interval) {
        for (let z = -radius; z <= radius; z += interval) {
          player.runCommand(`/execute if block ~${x} ~1 ~${z} air run setblock ~${x} ~1 ~${z} light_block["block_light_level"=15] replace`);
        }
      }
      clearItem();
    }

    function handleSummonAxesInArea(woodTypes, xRange, yRange, zRange) {
      player.runCommand(`clear @s ${item} 0 1`);
      let y = 0;

      system.runInterval(() => {
        if (y <= yRange) {
          for (let x = -xRange; x <= xRange; x++) {
            for (let z = -zRange; z <= zRange; z++) {
              for (const wood of woodTypes) {
                player.runCommand(`execute if block ~${x} ~${y} ~${z} minecraft:${wood}_log run summon sbs_mib:axe ~${x} ~${y} ~${z}`);
                player.runCommand(`execute if block ~${x} ~${y} ~${z} minecraft:${wood}_log run setblock ~${x} ~${y} ~${z} air destroy`);
              }
              player.runCommand(`execute if block ~${x} ~${y} ~${z} minecraft:wood run summon sbs_mib:axe ~${x} ~${y} ~${z}`);
              player.runCommand(`execute if block ~${x} ~${y} ~${z} minecraft:wood run setblock ~${x} ~${y} ~${z} air destroy`);
            }
          }
          y += 1;
        }
      }, TicksPerSecond * 0.25);
    }

    function handleTeleportWithIntervalAndSetBlock(block, teleportLimit, interval) {
      player.runCommand(`clear @s ${item} 0 1`);
      let y = 0;
      system.runInterval(() => {
        if (y <= teleportLimit) {
          if (!player.isSneaking) {
            player.runCommand(`execute unless block ~ ~2 ~ minecraft:bedrock run tp @s ~ ~1 ~`);
            player.runCommand(`execute unless block ~ ~-1 ~ minecraft:bedrock run setblock ~ ~-1 ~ minecraft:${block}`);
            y += 1;
          } else {
            y += teleportLimit;
          }
          player.runCommand(`/playsound block.scaffolding.place @s ~ ~ ~ 1 1 1`);
        }
      }, TicksPerSecond * interval);
    }

    function handleBatAndClear(entity, teleportRange, clearRange, tag) {
      player.runCommand(`tag @s add self`);
      player.runCommand(`summon minecraft:bat ^ ^ ^10`);
      player.runCommand(`tag @e[type=bat,c=1] add ${tag}`);
      player.runCommand(`execute as @e[tag=${tag}] at @s run tp @e[tag=!self,r=${teleportRange}] ~ ~ ~`);
      clearItem();
      system.runTimeout(() => {
        player.runCommand(`tp @e[type=bat,c=1,tag=${tag}] ~ ~-64 ~`);
        player.runCommand(`tag @s remove self`);
      }, TicksPerSecond * 3.0);
    }

    function handleWitherStructureLoad(particle) {
      player.runCommand(`playsound random.explode @a[r=32] ~ ~ ~ 1 1 1`);
      player.runCommand(`structure load "sbs_mib/sbs_mib_wither_bottle" ~-31 ~-31 ~-31`);
      player.runCommand(`particle ${particle}`);
      player.runCommand(`camerashake add @a[r=31] 1.5 1.5`);
      clearItem();
    }

    function handleTaggedSummonAndInterval(tag, tag_finish, entity, effect, level, duration, summonInterval, removeInterval1, removeInterval2) {
      player.addTag(tag);
      player.runCommand(`/effect @s ${effect} ${level} ${duration} true`);
      clearItem();
      system.runInterval(() => {
        if (player.hasTag(tag) && !player.hasTag(tag_finish)) {
          player.runCommand(`summon ${entity} ~ ~ ~`);
        }
      }, TicksPerSecond * summonInterval);
      system.runTimeout(() => {
        if (player.hasTag(tag)) {
          player.addTag(tag_finish);
          player.runCommand(`/effect @s ${effect} 0 0 true`);
        }
      }, TicksPerSecond * removeInterval1);
      system.runTimeout(() => {
        if (player.hasTag(tag_finish)) {
          player.removeTag(tag_finish);
          player.removeTag(tag);
          player.runCommand(`/effect @s ${effect} 0 0 true`);
        }
      }, TicksPerSecond * removeInterval2);
    }


    function clearItem() {
      player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
      player.runCommand(`clear @s ${item} 0 1`);
    }
  } catch (e) {
    console.log("Error inside itemStartUse afterEvent handle");
    console.log(e);
  }
});

world.afterEvents.projectileHitBlock.subscribe(data => {
  try {
    const { projectile } = data;
    if (projectile.isValid()) {
      const hitEntity = projectile.dimension.getEntities({ location: projectile.location, maxDistance: 2, closest: 1, excludeTypes: ["sbs_mib:mob_bottle", "minecraft:item"] })[0];
      if (hitEntity) mobBottleHit(projectile, hitEntity)
    }
  } catch (error) {
    console.log("Error inside entityHitBlock afterEvent handle");
    console.error(error);
  }
})
world.afterEvents.entityHitBlock.subscribe(data => {
  const { damagingEntity, blockFace, hitBlock: { location: { x, y, z } } } = data;

  function executeCommand(shiftX, shiftY, shiftZ) {
      damagingEntity.runCommand(`execute unless block ${x + shiftX} ${y + shiftY} ${z + shiftZ} bedrock run setblock ${x + shiftX} ${y + shiftY} ${z + shiftZ} air destroy`);
  }

  try {
      let mainhandItem = damagingEntity.getComponent('equippable').getEquipment(EquipmentSlot.Mainhand);
      if (mainhandItem && mainhandItem.typeId === "sbs_mib:ravager_ram" && damagingEntity.typeId === "minecraft:player") {
          const shifts = [1, 0, -1]; // Array to iterate through possible x or z shifts
          let yShifts = [0, -1, -2]; // Default y shifts for "Up" and "Down"

          if (blockFace === "North" || blockFace === "South" || blockFace === "East" || blockFace === "West") {
              yShifts = [1, 0, -1]; // Adjust y shifts for North, South, East, and West
          }

          shifts.forEach(shiftX => {
              shifts.forEach(shiftZ => {
                  yShifts.forEach(shiftY => {
                      executeCommand(shiftX, shiftY, shiftZ);
                  });
              });
          });

          const equipment = damagingEntity.getComponent("equippable");
          let ravagerRam = equipment.getEquipment(EquipmentSlot.Mainhand);
          const durability = ravagerRam.getComponent("durability");
          if (durability.damage <= 295) {
            durability.damage += 5;
          } else if (durability.damage === 296) {
            durability.damage += 4;
          } else if (durability.damage === 297) {
            durability.damage += 3;
          } else if (durability.damage === 298) {
            durability.damage += 2;
          } else if (durability.damage === 299) {
            durability.damage += 1;
          }
          if (ravagerRam.getComponent("durability").damage === 300) {
              ravagerRam = undefined;
              damagingEntity.runCommand(`/playsound random.break @s ~ ~ ~ 1 1 1`);
          }
          equipment.setEquipment(EquipmentSlot.Mainhand, ravagerRam);
      }
  } catch (error) {
      console.log("Error inside entityHitBlock afterEvent handle");
      console.error(error);
  }
});



world.afterEvents.entityHitEntity.subscribe(event => {
  try {
    const players = world.getAllPlayers();
    const damagingEntity = event.damagingEntity;
    const hitEntity = event.hitEntity;

    // Provera za zlatni paric
    if (damagingEntity.typeId == "minecraft:player" && damagingEntity.hasTag("sbs_mib_nearPiglinBruteUtil") && hitEntity.typeId != "minecraft:player") {
      hitEntity.runCommand(`particle sbs_mib:golden_particle ~ ~1 ~`);
      const goldNugget = new ItemStack("minecraft:gold_nugget", 1);
      hitEntity.dimension.spawnItem(goldNugget, hitEntity.location);
    }

    // Obrada za oružje Ravager Ram
    for (const player of players) {
      const force = player.getViewDirection();
      const mainhandItem = player.getComponent('equippable').getEquipment(EquipmentSlot.Mainhand);
      if (mainhandItem && mainhandItem.typeId === "sbs_mib:ravager_ram") {
        const familyMob = hitEntity.getComponent("minecraft:type_family");
        if (familyMob && familyMob.hasTypeFamily("mob")) {
          hitEntity.applyKnockback(force.x, force.z, 4.5, 1.0);
          const equipment = player.getComponent("equippable");
          const ravagerRam = equipment.getEquipment(EquipmentSlot.Mainhand);
          const durabilityComponent = ravagerRam.getComponent("durability");
          durabilityComponent.damage += 5;
          if (durabilityComponent.damage == 300) {
            equipment.setEquipment(EquipmentSlot.Mainhand, undefined);
            player.runCommand(`/playsound random.break @s ~ ~ ~ 1 1 1`);
          }
          else {
            equipment.setEquipment(EquipmentSlot.Mainhand, ravagerRam);
          }
        }
      }
    }
  } catch (error) {
    console.log("Greška u rukovanju događajem nakon što je entitet pogodio drugi entitet");
    console.error(error);
  }
});
