
// main imports
import { system } from "@minecraft/server";

// features
import * as util from "./util.js";
import { stunPlayer } from "../stun.js";

////////////////////////////////////////////////////////////////////////////////////////////////////

// list of durability reduction
const reduceDurability = {
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// Special attack for melee weapons
////////////////////////////////////////////////////////////////////////////////////////////////////

export function batBottle(event) {
  const player = event.source;
  util.shootRangedProjectile(4, "sbs_mib:bat_bottle_projectile", player);
  player.runCommand(`clear @s sbs_mib:bat_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function enderDragonBottle(event) {
  const player = event.source;
  const pet = player.dimension.spawnEntity("sbs_mib:ender_dragon_pet", player.location);
  const tameable = pet.getComponent("minecraft:tameable");
  tameable.tame(player);
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:ender_dragon_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function drownedBottle(event) {
  const player = event.source;
  util.shootMultiRangedProjectile(4, 10, 60, "sbs_mib:thrown_trident", player, 1.9);
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:drowned_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
export function blazeBottle(event) {
  const player = event.source;
  let attackCount = 0;
  const maxAttacks = 80;
  const delay = 2; // delay in ticks
  const shoot = () => {
    if (attackCount < maxAttacks) {
      util.shootRangedProjectile(7, "sbs_mib:blaze_bottle", player,1.28);
      attackCount++;
      system.runTimeout(shoot, delay); // Schedule the next attack
    }
  };
  shoot(); // Start the first attack
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:blaze_bottle_item -1 1`);
}
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
export function guardianBottle(event) {
  const player = event.source;
  let attackCount = 0;
  const maxAttacks = 80;
  const delay = 2; // delay in ticks
  const shoot = () => {
    if (attackCount < maxAttacks) {
      util.shootRangedProjectile(7, "sbs_mib:guardian_bottle", player,1);
      attackCount++;
      system.runTimeout(shoot, delay); // Schedule the next attack
    }
  };
  shoot(); // Start the first attack
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:guardian_bottle_item -1 1`);
}
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
export function ghastBottle(event) {
  const player = event.source;
  let attackCount = 0;
  const maxAttacks = 5;
  const delay = 5; // delay in ticks
  const shoot = () => {
    if (attackCount < maxAttacks) {
      util.shootRangedProjectile(5, "sbs_mib:ghast_bottle", player,1.28);
      attackCount++;
      system.runTimeout(shoot, delay); // Schedule the next attack
    }
  };
  shoot(); // Start the first attack
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:ghast_bottle_item -1 1`);
}
////////////////////////////////////////////////////////////////////////////////////////////////////

export function skeletonHorseBottle(event) {
  const player = event.source;

  let attackCount = 0;
  const maxAttacks = 20;
  const delay = 5 // delay in ticks
  const dealDamage = () => {
    if (attackCount < maxAttacks) {
      player.runCommand(`summon minecraft:lightning_bolt ~${getRandomOffset()} ~ ~${getRandomOffset()}`)
      attackCount++;
      system.runTimeout(dealDamage, delay); // Schedule the next attack
    }
  };
  dealDamage(); // Start the first attack
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:skeleton_horse_bottle -1 1`);
}

function getRandomOffset() {
  return (Math.random() < 0.5 ? -1 : 1) * (3 + Math.random() * 3);
}
////////////////////////////////////////////////////////////////////////////////////////////////////

export function mooshroomBottle(event) {
  const player = event.source;
  for (let y = -5; y <= 0; y++) {
    for (let x = -4; x <= 4; x++) {
      for (let z = -4; z <= 4; z++) {
        player.runCommand(`execute positioned ~${x} ~${y} ~${z} unless block ~~~ air run setblock ~~~ mycelium replace`);
      }
    }
  }
  const mushroomType = Math.random() < 0.75 ? "red_mushroom" : "brown_mushroom";
  player.runCommand(`setblock ~1 ~ ~ ${mushroomType}`);
  player.runCommand(`setblock ~ ~ ~ ${mushroomType}`);
  player.runCommand(`setblock ~ ~ ~2 ${mushroomType}`);
  player.runCommand(`setblock ~ ~ ~-1 ${mushroomType}`);
  player.runCommand(`summon sbs_mib:mooshroom_bottle_util ~~~`);
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:mooshroom_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function creakingBottle(event) {
  const player = event.source;
  const entities = util.getEntitiesFromRadius(player, 10);
  for (const entity of entities) {
    const loc = entity.location;
    if (entity.typeId == "minecraft:player") {
      stunPlayer(entity, 20);
      entity.runCommand(`summon sbs_mib:creaking_bottle_util ~~~`);
    } else if (entity.typeId != "sbs_mib:creaking_bottle_util" && entity.typeId != "minecraft:item") {
      if (!entity.getComponent("is_tamed")?.isValid()) {
        entity.runCommand(`effect @s slowness 20 255 true`);
        entity.runCommand(`playsound mob.creaking.attack @a[r=16] ~ ~ ~ 1 1 1`);
        entity.runCommand(`summon sbs_mib:creaking_bottle_util ${loc.x}${loc.y}${loc.z}`);
      }
    }
  }
  player.runCommand(`effect @s resistance 20 5 true`);
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:creaking_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function ocelotBottle(event) {
  const player = event.source;
  player.runCommand(`effect @s speed 60 3 true`);
  player.runCommand(`effect @s jump_boost 60 3 true`);
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:ocelot_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function huskBottle(event) {
  const player = event.source;
  let iterationCount = 0;
  const maxIterations = 80;
  const delay = 5; // delay in ticks
  const replaceBlocks = () => {
    if (iterationCount < maxIterations) {
      // replace water
      player.runCommand(`execute rotated ~ 0 positioned ^^^2 run fill ~-2 ~-30 ~-2 ~2 ~-1 ~2 sand replace seagrass`);
      player.runCommand(`execute rotated ~ 0 positioned ^^^2 run fill ~-2 ~-30 ~-2 ~2 ~-1 ~2 sand replace kelp`);
      player.runCommand(`execute rotated ~ 0 positioned ^^^2 run fill ~-2 ~-30 ~-2 ~2 ~-1 ~2 sand replace water`);
      player.runCommand(`execute rotated ~ 0 positioned ^^^2 run fill ~-2 ~-30 ~-2 ~2 ~-1 ~2 sand replace flowing_water`);
      // destroy crops
      for (let y = -2; y <= 1; y++) {
        for (let x = -2; x <= 2; x++) {
          for (let z = -2; z <= 2; z++) {
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:beetroot run setblock ~~~ air destroy`);
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:carrots run setblock ~~~ air destroy`);
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:pitcher_crop run setblock ~~~ air destroy`);
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:potatoes run setblock ~~~ air destroy`);
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:sweet_berry_bush run setblock ~~~ air destroy`);
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:torchflower run setblock ~~~ air destroy`);
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:torchflower_crop run setblock ~~~ air destroy`);
            player.runCommand(`execute positioned ~${x} ~${y} ~${z} if block ~~~ minecraft:wheat run setblock ~~~ air destroy`);
          }
        }
      }
      iterationCount++;
      system.runTimeout(replaceBlocks, delay); // Schedule the next replace
    }
  };
  replaceBlocks(); // Start the first replace


  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:husk_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function piglinBruteBottle(event) {
  const player = event.source;
  util.throwProjectile("sbs_mib:piglin_brute_bottle_projectile", player);
  player.playSound("sbs_mib:use_bottle", { pitch: 1, volume: 1 });
  player.runCommand(`clear @s sbs_mib:piglin_brute_bottle -1 1`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
