
import { system } from "@minecraft/server";
import { bVector3 } from "../lib/better_vectors.js"

export function pullEntities(event) {
  const source = event.sourceEntity;
  source.runCommand(`tp @e[r=10] ~~~`);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function shootMultiRangedProjectile(power, num_of_proj, angle, projectile, player) {
  const location = player.location;
  const rotation = player.getRotation();
  const pitch = rotation.x;
  const yaw = rotation.y;

  const radYaw = (yaw * Math.PI) / 180;
  const radPitch = (pitch * Math.PI) / 180;

  // Calculate the position 1 block in front of the player
  const offsetX = -Math.sin(radYaw) * Math.cos(radPitch) * 2;
  const offsetY = -Math.sin(radPitch);
  const offsetZ = Math.cos(radYaw) * Math.cos(radPitch) * 2;

  const spawnLocation = {
    x: location.x + offsetX,
    y: location.y + 1.5 + offsetY,  // Adjust Y to make the spawn a little above the player
    z: location.z + offsetZ,
  };

  // Determine angle step between each projectile
  const angleStep = angle / Math.max(num_of_proj - 1, 1); // Avoid division by zero
  const startAngle = -angle / 2; // Start at negative half of the cone angle

  for (let i = 0; i < num_of_proj; i++) {
    const offsetYaw = startAngle + i * angleStep; // Spread projectiles within the cone
    const radOffsetYaw = (offsetYaw * Math.PI) / 180;

    // Calculate velocity with yaw offset
    const velocity = {
      x: power * (-Math.sin(radYaw + radOffsetYaw) * Math.cos(radPitch)),
      y: (1.5 + power / 10) * (-Math.sin(radPitch)),
      z: power * (Math.cos(radYaw + radOffsetYaw) * Math.cos(radPitch)),
    };

    // Spawn the projectile 1 block in front of the player
    const arrow = player.dimension.spawnEntity(projectile, spawnLocation);
    // Normalize the yaw
    arrow.setRotation({ x: 0, y: -yaw });

    // Apply velocity
    const projectileComp = arrow.getComponent("minecraft:projectile");
    projectileComp?.shoot(velocity);
    arrow.setRotation({ x: 0, y: -yaw });

    const fixRot = () => {
      if (!arrow?.isValid()) return;
      if (!arrow.isOnGround) {
        const dir = bVector3.fromVector3(arrow.getVelocity());
        const rot = dir.angles(true);
        if (dir.magnitude() > 0.5) {
          arrow.setProperty(`sbs_mw:rotation_x`, rot.theta);
          arrow.setProperty(`sbs_mw:rotation_y`, rot.phi);
        }
      }
      system.runTimeout(fixRot, 1);
    }
    fixRot();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function shootRangedProjectile(power, projectile, player, height = 1.5) {
  const location = player.location;
  const rotation = player.getRotation();
  const pitch = rotation.x;
  const yaw = rotation.y;

  // Calculate velocity based on the player's rotation
  const radYaw = (yaw * Math.PI) / 180; // Convert yaw to radians
  const radPitch = (pitch * Math.PI) / 180; // Convert pitch to radians
  const velocity = {
    x: power * (-Math.sin(radYaw) * Math.cos(radPitch)),
    y: (1 + power / 1.5) * (-Math.sin(radPitch)),
    z: power * (Math.cos(radYaw) * Math.cos(radPitch)),
  };

  // Spawn the arrow
  const arrow = player.dimension.spawnEntity(projectile, {
    x: location.x,
    y: location.y + height,
    z: location.z,
  });
  // Normalize the yaw
  arrow.setRotation({ x: 0, y: -yaw });

  // Get the projectile component and apply velocity
  const projectileComp = arrow.getComponent("minecraft:projectile");
  projectileComp?.shoot(velocity);
  arrow.setRotation({ x: 0, y: -yaw });

  const fixRot = () => {
    if (!arrow?.isValid()) return;
    if (!arrow.isOnGround) {
      const dir = bVector3.fromVector3(arrow.getVelocity());
      const rot = dir.angles(true);
      if (dir.magnitude() > 0.5) {
        arrow.setProperty(`sbs_mw:rotation_x`, rot.theta);
        arrow.setProperty(`sbs_mw:rotation_y`, rot.phi);
      }
    }
    system.runTimeout(fixRot, 1);
  }
  fixRot();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function throwProjectile(projectile, player) {
  const location = player.location;
  const rotation = player.getRotation();
  const pitch = rotation.x;
  const yaw = rotation.y;

  // Calculate velocity based on the player's rotation
  const radYaw = (yaw * Math.PI) / 180; // Convert yaw to radians
  const radPitch = (pitch * Math.PI) / 180; // Convert pitch to radians
  const velocity = {
    x: 1.1 * (-Math.sin(radYaw) * Math.cos(radPitch)),
    y: 1.2 * (-Math.sin(radPitch)),
    z: 1.1 * (Math.cos(radYaw) * Math.cos(radPitch)),
  };

  // Spawn the arrow
  const arrow = player.dimension.spawnEntity(projectile, {
    x: location.x,
    y: location.y + 2,
    z: location.z,
  });
  // Normalize the yaw
  arrow.setRotation({ x: 0, y: -yaw });

  // Get the projectile component and apply velocity
  const projectileComp = arrow.getComponent("minecraft:projectile");
  projectileComp?.shoot(velocity);
  arrow.setRotation({ x: 0, y: -yaw });

  const fixRot = () => {
    if (!arrow?.isValid()) return;
    if (!arrow.isOnGround) {
      const dir = bVector3.fromVector3(arrow.getVelocity());
      const rot = dir.angles(true);
      if (dir.magnitude() > 0.5) {
        arrow.setProperty(`sbs_mw:rotation_x`, rot.theta);
        arrow.setProperty(`sbs_mw:rotation_y`, rot.phi);
      }
    }
    system.runTimeout(fixRot, 1);
  }
  fixRot();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function shootMeleeProjectile(projectile, player) {
  const location = player.location;
  const rotation = player.getRotation();
  const pitch = rotation.x;
  const yaw = rotation.y;

  // Calculate velocity based on the player's rotation
  const radYaw = (yaw * Math.PI) / 180; // Convert yaw to radians
  const radPitch = (pitch * Math.PI) / 180; // Convert pitch to radians
  const velocity = {
    x: 2.8 * (-Math.sin(radYaw) * Math.cos(radPitch)),
    y: 2.8 * (-Math.sin(radPitch)),
    z: 2.8 * (Math.cos(radYaw) * Math.cos(radPitch)),
  };

  // Spawn the arrow
  const arrow = player.dimension.spawnEntity(projectile, {
    x: location.x,
    y: location.y + 1.5,
    z: location.z,
  });
  // Normalize the yaw
  arrow.setRotation({ x: 0, y: -yaw });

  // Get the projectile component and apply velocity
  const projectileComp = arrow.getComponent("minecraft:projectile");
  projectileComp?.shoot(velocity);
  arrow.setRotation({ x: 0, y: -yaw });

  const fixRot = () => {
    if (!arrow?.isValid()) return;
    if (!arrow.isOnGround) {
      const dir = bVector3.fromVector3(arrow.getVelocity());
      const rot = dir.angles(true);
      if (dir.magnitude() > 0.5) {
        arrow.setProperty(`sbs_mw:rotation_x`, rot.theta);
        arrow.setProperty(`sbs_mw:rotation_y`, rot.phi);
      }
    }
    system.runTimeout(fixRot, 1);
  }
  fixRot();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export const rand = (min, max) => Math.random() * (max - min) + min;

////////////////////////////////////////////////////////////////////////////////////////////////////

export function getEntitiesFromCone(source, r, angle) {
  const dim = source.dimension;
  const location = source.location;
  const direction = source.getViewDirection();
  const angleRadians = Math.PI * angle / 180; // Convert angle to radians
  const cosAngle = Math.cos(angleRadians); // Pre-calculate cos(angle)
  const entitiesInCone = []; // Array to hold entities within the cone

  // Get entities within the specified radius
  const entitiesInRange = dim.getEntities({
    location: location,
    maxDistance: r,
  });

  // Calculate the length of the view direction vector (used for normalization)
  const directionLength = Math.sqrt(direction.x ** 2 + direction.y ** 2 + direction.z ** 2);
  const normalizedDirection = {
    x: direction.x / directionLength,
    y: direction.y / directionLength,
    z: direction.z / directionLength,
  };

  // Iterate through each entity to check if it's within the cone
  for (const entity of entitiesInRange) {
    const entityLocation = entity.location;
    const vectorToEntity = {
      x: entityLocation.x - location.x,
      y: entityLocation.y - location.y,
      z: entityLocation.z - location.z,
    };

    // Skip entities that are out of the radius
    const distanceSquared = vectorToEntity.x ** 2 + vectorToEntity.y ** 2 + vectorToEntity.z ** 2;
    if (distanceSquared > r * r) continue; // Entity is out of radius

    // Normalize the vector to the entity
    const entityLength = Math.sqrt(distanceSquared);
    const normalizedEntityVector = {
      x: vectorToEntity.x / entityLength,
      y: vectorToEntity.y / entityLength,
      z: vectorToEntity.z / entityLength,
    };

    // Dot product between the direction vector and the entity vector
    const dotProduct = normalizedDirection.x * normalizedEntityVector.x +
      normalizedDirection.y * normalizedEntityVector.y +
      normalizedDirection.z * normalizedEntityVector.z;

    // If the dot product is greater than or equal to cos(angle), the entity is within the cone
    if (dotProduct >= cosAngle) {
      entitiesInCone.push(entity);
    }
  }

  return entitiesInCone;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function getEntitiesFromRadius(source, r) {
  const dim = source.dimension;
  const options = {
    excludeNames: source.name ? [source.name] : [],
    location: source.location,
    maxDistance: r,
  };
  return dim.getEntities(options);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function getTargetEntity(player) {
  const rayResult = player.getEntitiesFromViewDirection({ maxDistance: 6 });
  if (rayResult.length > 0) return rayResult[0].entity;
  return null;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function damageItem(player, damageAmount) {
  const equip = player.getComponent('equippable');
  const item = equip.getEquipment('Mainhand');
  if (!item) return;
  const durability = item.getComponent('durability');
  if (durability) {
    try {
      durability.damage += damageAmount;
      equip.setEquipment('Mainhand', item);
    } catch {
      equip.setEquipment('Mainhand', undefined);
    }
  }

}

////////////////////////////////////////////////////////////////////////////////////////////////////